package require XOTcl
namespace import ::xotcl::*

package require rete
namespace import ::rete::*

::rete::WorkingMemory workingMem

namespace eval ::measurement {
	#
	# Simple Model...
	#
	
	Class Company -slots {
	  Attribute name
	  Attribute location
	}
	
	Class Person -slots {
	  Attribute name 
	  Attribute age
	  Attribute salary
	  Attribute employed -type ::measurement::Company
	}
	
	
	#
	# ... with a few facts ....
	#
	Company c1 -name KM -location Vienna 
	Company c2 -name AS -location US
	Company c3 -name WU -location Vienna

	
	Person p1 -name Bernd -age 31 -salary 100 -employed ::measurement::c1
	Person p2 -name Gustaf -age 77 -salary 120 -employed ::measurement::c3
	Person p3 -name Franz -age 22 -salary 80 -employed ::measurement::c3
	Person p4 -name Jeff -age 55 -salary 120 -employed ::measurement::c2

	for { set i 1 } { $i <= 1000 } { incr i } {
		[Company new -name temp -location Vienna]
	}
	puts "number Companies: [llength [Company info instances]]"
	
	# ok, we are done. Define a materialized View
	# Workplace...
	#
	set query {[$p employed] eq $c && [$c location] eq "Vienna"}
	puts "\nrun Query1: '$query'"
	
	set time_first [time {	 
		View Workplace as {
		  forall {::measurement::Person p ::measurement::Company c} {
		    my set name [$p name]
			my set company [$c name]  
		    my set location [$c location]
		  } where {
		     [$p employed] eq $c && [$c location] eq "Vienna"  
		  }
		}
	}]
	
	#
	# Show the results
	#
	puts "print results of Query 1"
	foreach w [Workplace info instances] {
	  puts  "\t $w company '[$w set company]' location '[$w set location]' employee '[$w set name]'"
	}
	
	
	#
	# rerun
	#
	
	set time_second [time {
		View Workplace as {
		  forall {::measurement::Person p ::measurement::Company c} {
		    my set name [$p name]
			my set company [$c name]  
		    my set location [$c location]
		  } where {
		    [$p employed] eq $c && [$c location] eq "Vienna" 
		  }
		}
	}]
	puts  stderr "First run: $time_first"
	puts  stderr "Rerun tooks: $time_second"
}